package mi.hdm.recipes;

import mi.hdm.mealPlan.MealPlan;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.time.LocalDate;
import java.util.Map;

public class NutritionCalculator {

    private static final Logger log = LogManager.getLogger(NutritionCalculator.class);

    /**
     * Method to calculate the nutrition table for a recipe out of its ingredients.
     *
     * @return the nutrition table for this recipe based on its ingredients. All nutrition values are added up.
     */
    public static NutritionTable calculateNutritionTable(Map<RecipeComponent, Integer> ingredients) {
        if (ingredients == null || ingredients.isEmpty()) {
            log.info("Map ingredients is null or empty. Therefore new nutritionTable() was generated.");
            return NutritionTable.empty();
        }

        double totalCals = 0,
                totalCarbs = 0,
                totalFats = 0,
                totalProteins = 0,
                totalFibers = 0,
                totalSalt = 0;

        for (RecipeComponent entry : ingredients.keySet()) {
            final Map<Nutrition, Double> nutritionTable = entry.getNutritionTable().getTable();

            int divisor;
            if (entry.getMeasurement() == Measurement.PIECE || entry.getMeasurement() == Measurement.PINCH) {
                divisor = 1;
            } else {
                divisor = 100;
            }
            totalCals += nutritionTable.get(Nutrition.CALORIES) * ingredients.get(entry) / divisor;
            totalCarbs += nutritionTable.get(Nutrition.CARBS) * ingredients.get(entry) / divisor;
            totalFats += nutritionTable.get(Nutrition.FAT) * ingredients.get(entry) / divisor;
            totalProteins += nutritionTable.get(Nutrition.PROTEINS) * ingredients.get(entry) / divisor;
            totalFibers += nutritionTable.get(Nutrition.FIBERS) * ingredients.get(entry) / divisor;
            totalSalt += nutritionTable.get(Nutrition.SALT) * ingredients.get(entry) / divisor;
        }
        log.info("NutritionTable calculated successfully.");
        return new NutritionTable(totalCals, totalCarbs, totalFats, totalProteins, totalFibers, totalSalt);
    }

    //TODO: implement calcuating nutrition table from meal plan and List<Recipe>
    public static NutritionTable calculateNutritionTable(MealPlan mealPlan) {

        Map<LocalDate, Recipe> plan = mealPlan.getAllRecipesFromPlan();

        double totalCals = 0,
                totalCarbs = 0,
                totalFats = 0,
                totalProteins = 0,
                totalFibers = 0,
                totalSalt = 0;

        for (Recipe r : plan.values()) {
            final Map<Nutrition, Double> nutritionTable = r.getNutritionTable().getTable();
            totalCals += nutritionTable.get(Nutrition.CALORIES);
            totalCarbs += nutritionTable.get(Nutrition.CARBS);
            totalFats += nutritionTable.get(Nutrition.FAT);
            totalProteins += nutritionTable.get(Nutrition.PROTEINS);
            totalFibers += nutritionTable.get(Nutrition.FIBERS);
            totalSalt += nutritionTable.get(Nutrition.SALT);
        }
        log.info("NutritionTable calculated successfully.");

        return new NutritionTable(totalCals, totalCarbs, totalFats, totalProteins, totalFibers, totalSalt);
    }
}
